@@ -9,7 +9,7 @@ fn main() {
         // Generate a simple sine wave
         (0..RATE * DURATION_SECS).map(|i| {
             let t = i as f32 / RATE as f32;
-            (t * 500.0 * 2.0 * std::f32::consts::PI).sin() * 80.0
+            (t * 500.0 * 2.0 * core::f32::consts::PI).sin() * 80.0
         }),
     );
     let (mut scene_handle, scene) = oddio::split(oddio::SpatialScene::new(RATE, 0.1));
@@ -18,7 +18,6 @@ fn main() {
         oddio::SpatialOptions {
             position: [-SPEED, 10.0, 0.0].into(),
             velocity: [SPEED, 0.0, 0.0].into(),
-            max_distance: 1000.0,
             radius: 0.1,
         },
     );
@@ -29,7 +28,7 @@ fn main() {
         bits_per_sample: 16,
         sample_format: hound::SampleFormat::Int,
     };
-    let mut writer = hound::WavWriter::create("a.wav", spec).unwrap();
+    let mut writer = hound::WavWriter::create("offline.wav", spec).unwrap();
 
     for _ in 0..(RATE * DURATION_SECS / BLOCK_SIZE as u32) {
         let mut block = [[0.0; 2]; BLOCK_SIZE];
