@@ -1,4 +1,4 @@
-use crate::Signal;
+use crate::{Seek, Signal};
 
 /// A constant signal, useful for testing
 pub struct Constant<T>(pub T);
@@ -17,3 +17,7 @@ impl<T: Clone> Signal for Constant<T> {
         out.fill(self.0.clone());
     }
 }
+
+impl<T: Clone> Seek for Constant<T> {
+    fn seek(&self, _: f32) {}
+}
