@@ -23,6 +23,9 @@
 //! handle.control::<oddio::Spatial<_>, _>().set_motion(position, velocity, false);
 //! ```
 //!
+//! To get started, review [the `examples`
+//! subdirectory](https://github.com/Ralith/oddio/tree/main/examples) in the crate source.
+//!
 //! Key primitives:
 //! - [`Frames`] stores static audio data, which can be played with a [`FramesSignal`]
 //! - [`Mixer`] allows multiple signals to be played concurrently and controlled during playback
@@ -30,11 +33,18 @@
 //! - [`Handle`] allows control of a signal while it's playing, from a mixer or [`split`]
 //! - [`run`] writes frames from a [`Signal`] into an output buffer
 
+#![allow(unused_imports)]
 #![warn(missing_docs)]
+#![no_std]
+
+extern crate alloc;
+#[cfg(not(feature = "no_std"))]
+extern crate std;
 
 mod adapt;
 mod constant;
 mod cycle;
+mod downmix;
 mod filter;
 mod frame;
 mod frames;
@@ -53,10 +63,12 @@ mod spsc;
 mod stop;
 mod stream;
 mod swap;
+mod tanh;
 
 pub use adapt::{Adapt, AdaptOptions};
 pub use constant::Constant;
 pub use cycle::Cycle;
+pub use downmix::Downmix;
 pub use filter::*;
 pub use frame::Frame;
 pub use frames::*;
@@ -72,6 +84,7 @@ pub use speed::{Speed, SpeedControl};
 pub use stop::*;
 pub use stream::{Stream, StreamControl};
 pub use swap::Swap;
+pub use tanh::Tanh;
 
 /// Unitless instantaneous sound wave amplitude measurement
 pub type Sample = f32;
@@ -89,13 +102,13 @@ pub fn run<S: Signal + ?Sized>(signal: &S, sample_rate: u32, out: &mut [S::Frame
 /// The [`Handle`] can be used to control the signal concurrent with the [`SplitSignal`] being
 /// played
 pub fn split<S: Signal>(signal: S) -> (Handle<S>, SplitSignal<S>) {
-    let signal = std::sync::Arc::new(signal);
+    let signal = alloc::sync::Arc::new(signal);
     let handle = unsafe { Handle::from_arc(signal.clone()) };
     (handle, SplitSignal(signal))
 }
 
 /// A concurrently controlled [`Signal`]
-pub struct SplitSignal<S: ?Sized>(std::sync::Arc<S>);
+pub struct SplitSignal<S: ?Sized>(alloc::sync::Arc<S>);
 
 impl<S> Signal for SplitSignal<S>
 where
@@ -119,9 +132,9 @@ unsafe impl<S: ?Sized> Send for SplitSignal<S> {}
 ///
 /// Useful for adapting output buffers obtained externally.
 pub fn frame_stereo(xs: &mut [Sample]) -> &mut [[Sample; 2]] {
-    unsafe { std::slice::from_raw_parts_mut(xs.as_mut_ptr() as _, xs.len() / 2) }
+    unsafe { core::slice::from_raw_parts_mut(xs.as_mut_ptr() as _, xs.len() / 2) }
 }
 
 fn flatten_stereo(xs: &mut [[Sample; 2]]) -> &mut [Sample] {
-    unsafe { std::slice::from_raw_parts_mut(xs.as_mut_ptr() as _, xs.len() * 2) }
+    unsafe { core::slice::from_raw_parts_mut(xs.as_mut_ptr() as _, xs.len() * 2) }
 }
