@@ -1,4 +1,4 @@
-use crate::{Filter, Frame, Signal};
+use crate::{math::Float, Filter, Frame, Seek, Signal};
 
 /// Smoothly maps a signal of any range into (-1, 1)
 ///
@@ -8,6 +8,8 @@ use crate::{Filter, Frame, Signal};
 /// when spatial signals are very near by, audio can get arbitrarily loud. Because surprisingly loud
 /// audio can be disruptive and even damaging, it can be useful to limit the output range, but
 /// simple clamping introduces audible artifacts.
+///
+/// See also [`Tanh`](crate::Tanh), which distorts quiet sounds less, and loud sounds more.
 pub struct Reinhard<T>(T);
 
 impl<T> Reinhard<T> {
@@ -48,3 +50,13 @@ impl<T> Filter for Reinhard<T> {
         &self.0
     }
 }
+
+impl<T> Seek for Reinhard<T>
+where
+    T: Signal + Seek,
+    T::Frame: Frame,
+{
+    fn seek(&self, seconds: f32) {
+        self.0.seek(seconds);
+    }
+}
