@@ -1,6 +1,6 @@
-use std::{cell::Cell, f32::consts::TAU};
+use core::{cell::Cell, f32::consts::TAU};
 
-use crate::{Sample, Signal};
+use crate::{math::Float, Sample, Seek, Signal};
 
 /// A trivial [`Signal`] that produces a sine wave of a particular frequency, forever
 pub struct Sine {
@@ -40,3 +40,9 @@ impl Signal for Sine {
         self.seek_to(interval * out.len() as f32);
     }
 }
+
+impl Seek for Sine {
+    fn seek(&self, seconds: f32) {
+        self.seek_to(seconds);
+    }
+}
